class DuplicatePeerError(Exception):
    def __init__(self, name: str):
        super().__init__(f"peer already exists: {name}")


class UnknownPeerError(Exception):
    def __init__(self, name: str):
        super().__init__(f"unknown peer: {name}")


class UnknownSiteError(Exception):
    def __init__(self, name: str):
        super().__init__(f"unknown site: {name}")


class FreeAddressError(Exception):
    def __init__(self, protocol: str):
        super().__init__(f"no free {protocol} address")


class ConfigVersionError(Exception):
    def __init__(self, version: int, current_version: int):
        super().__init__(
            f"config version {version} is newer than supported version "
            f"{current_version}"
        )


class MissingMigration(Exception):
    def __init__(self, from_version: int, to_version: int):
        super().__init__(
            f"missing migration for config version {from_version} -> {to_version}"
        )



from __future__ import annotations

from pathlib import Path

from typer import Option

DEFAULT_CONFIG_PATH = Path("wgmgr.yml")

OPTION_CONFIG_PATH = Option(
    DEFAULT_CONFIG_PATH,
    "-c",
    "--config",
    envvar="WGMGR_CONFIG",
    help="Path of the config file.",
)

OPTION_PORT = Option(None, "-p", "--port", help="Port for WireGuard.")

OPTION_IPV4_NETWORK = Option(
    None,
    "-4",
    "--ipv4",
    help="IPv4 network in CIDR notation.",
)

OPTION_IPV6_NETWORK = Option(
    None,
    "-6",
    "--ipv6",
    help="IPv6 network in CIDR notation.",
)


OPTION_IPV4_ADDRESS = Option(
    None,
    "-4",
    "--ipv4",
    help="IPv4 address.",
)

OPTION_IPV6_ADDRESS = Option(
    None,
    "-6",
    "--ipv6",
    help="IPv6 address.",
)



from __future__ import annotations

import logging
from ipaddress import IPv4Address, IPv4Network, IPv6Address, IPv6Network
from typing import cast

from wgmgr.base import MainConfigBase
from wgmgr.util import AssignableIPv4, AssignableIPv6

LOGGER = logging.getLogger(__name__)


def set_default_port(self: MainConfigBase, port: int):
    if port == self.default_port:
        LOGGER.warn("port already set to %d, nothing to do", port)
        return

    self.default_port = port
    for peer in self.peers:
        if peer.port.auto:
            LOGGER.info("set port to %d for peer %s", port, peer.name)
            peer.port.set_auto(port)


def set_ipv4_network(self: MainConfigBase, network: IPv4Network):
    self.ipv4_network = network
    for peer in self.peers:
        if peer.ipv4:
            if peer.ipv4.address in self.ipv4_network:
                continue
            if not peer.ipv4.auto:
                LOGGER.warn(
                    f"udpating IPv4 address of peer {peer} as it is not "
                    "included in the new network"
                )
            peer.ipv4.set_auto(cast(IPv4Address, self.get_next_ipv4()))
        else:
            peer.ipv4 = AssignableIPv4(cast(IPv4Address, self.get_next_ipv4()), True)


def set_ipv6_network(self: MainConfigBase, network: IPv6Network):
    self.ipv6_network = network
    for peer in self.peers:
        if peer.ipv6:
            if peer.ipv6.address in self.ipv6_network:
                continue
            if not peer.ipv6.auto:
                LOGGER.warn(
                    f"udpating manual IPv6 address of peer {peer} as it is not "
                    "included in the new network"
                )
            peer.ipv6.set_auto(cast(IPv6Address, self.get_next_ipv6()))
        else:
            peer.ipv6 = AssignableIPv6(cast(IPv6Address, self.get_next_ipv6()), True)



from __future__ import annotations

from typing import Any

from wgmgr import keygen


class PointToPointConfig:
    def __init__(
        self,
        name1: str,
        name2: str,
        endpoint1: str | None = None,
        endpoint2: str | None = None,
    ):
        self.peer1_name = name1
        self.peer2_name = name2
        self.peer1_endpoint = endpoint1
        self.peer2_endpoint = endpoint2
        self.preshared_key = keygen.generate_psk()

    def serialize(self) -> dict[str, Any]:
        return {
            "peer1": {
                "name": self.peer1_name,
                "endpoint": self.peer1_endpoint if self.peer1_endpoint else None,
            },
            "peer2": {
                "name": self.peer2_name,
                "endpoint": self.peer2_endpoint if self.peer2_endpoint else None,
            },
            "preshared_key": self.preshared_key,
        }

    @staticmethod
    def deserialize(data: dict[str, Any]) -> PointToPointConfig:
        config = PointToPointConfig(
            data["peer1"]["name"],
            data["peer2"]["name"],
            data["peer1"]["endpoint"],
            data["peer2"]["endpoint"],
        )
        config.preshared_key = data["preshared_key"]
        return config



from __future__ import annotations

import logging
import os
from ipaddress import IPv4Network, IPv6Network
from pathlib import Path
from typing import cast

import yaml

import wgmgr.operations.config as ops_config
import wgmgr.operations.p2p as ops_p2p
import wgmgr.operations.peer as ops_peer
from wgmgr import keygen
from wgmgr.base import MainConfigBase

LOGGER = logging.getLogger(__name__)


class MainConfig(MainConfigBase):
    add_peer = ops_peer.add_peer
    remove_peer = ops_peer.remove_peer
    set_default_port = ops_config.set_default_port
    set_ipv4_network = ops_config.set_ipv4_network
    set_ipv6_network = ops_config.set_ipv6_network
    generate_peer_config = ops_peer.generate_peer_config
    add_p2p = ops_p2p.add_p2p

    def __init__(
        self,
        default_port: int,
        ipv4_network: IPv4Network | None = None,
        ipv6_network: IPv6Network | None = None,
    ):
        super().__init__(default_port, ipv4_network, ipv6_network)

    def regenerate_all_keys(self):
        for peer in self.peers:
            self.regenerate_keys_for_peer(peer.name)

    def regenerate_keys_for_peer(self, name: str):
        peer = self.get_peer(name)

        peer.private_key = keygen.generate_private_key()
        peer.public_key = keygen.generate_public_key(peer.private_key)

        for p2p in self.point_to_point:
            if p2p.peer1_name == name:
                p2p.preshared_key = keygen.generate_psk()
            elif p2p.peer2_name == name:
                p2p.preshared_key = keygen.generate_psk()

    def save(self, path: Path):
        with os.fdopen(
            os.open(path, os.O_WRONLY | os.O_CREAT | os.O_TRUNC, mode=0o600), "w"
        ) as fptr:
            yaml.dump(self.serialize(), fptr, yaml.CDumper)

    @staticmethod
    def load(path: Path) -> MainConfig:
        with os.fdopen(os.open(path, os.O_RDONLY, mode=0o600), "r") as fptr:
            obj = cast(
                MainConfig, MainConfigBase.deserialize(yaml.load(fptr, yaml.CLoader))
            )
            obj.__class__ = MainConfig
            return obj



import subprocess


def generate_private_key() -> str:
    return subprocess.check_output(["wg", "genkey"]).decode().strip()


def generate_public_key(private_key: str) -> str:
    return (
        subprocess.check_output(["wg", "pubkey"], input=private_key.encode())
        .decode()
        .strip()
    )


def generate_psk() -> str:
    return subprocess.check_output(["wg", "genpsk"]).decode().strip()



from __future__ import annotations

from ipaddress import IPv4Address, IPv4Network, IPv6Address, IPv6Network
from logging import getLogger
from typing import Any

from wgmgr.error import ConfigVersionError, FreeAddressError, UnknownPeerError
from wgmgr.migrations import load_migration
from wgmgr.p2p import PointToPointConfig
from wgmgr.peer import PeerConfig
from wgmgr.site import Site

CURRENT_CONFIG_VERSION = 1


LOGGER = getLogger(__name__)


class MainConfigBase:
    def __init__(
        self,
        default_port: int,
        ipv4_network: IPv4Network | None = None,
        ipv6_network: IPv6Network | None = None,
    ):
        self.config_version = CURRENT_CONFIG_VERSION
        self.ipv4_network = ipv4_network
        self.ipv6_network = ipv6_network
        self.default_port = default_port
        self.peers: list[PeerConfig] = []
        self.point_to_point: list[PointToPointConfig] = []
        self.sites: list[Site] = []

    @staticmethod
    def migrate(data: dict[str, Any]) -> dict[str, Any]:
        while int(data["version"]) < CURRENT_CONFIG_VERSION:
            LOGGER.info(
                "migrate config from version %d to %d",
                int(data["version"]),
                int(data["version"]) + 1,
            )
            data = load_migration(data["version"])(data)
        return data

    def get_peer(self, name: str) -> PeerConfig:
        for peer in self.peers:
            if peer.name == name:
                return peer
        raise UnknownPeerError(name)

    def get_used_ipv4_addresses(self) -> list[IPv4Address]:
        result: list[IPv4Address] = []
        for peer in self.peers:
            if peer.ipv4:
                result.append(peer.ipv4.address)
        return result

    def get_used_ipv6_addresses(self) -> list[IPv6Address]:
        result: list[IPv6Address] = []
        for peer in self.peers:
            if peer.ipv6:
                result.append(peer.ipv6.address)
        return result

    def get_next_ipv4(self) -> IPv4Address | None:
        if not self.ipv4_network:
            return None

        used = self.get_used_ipv4_addresses()
        for address in self.ipv4_network.hosts():
            if address in used:
                continue
            return address

        raise FreeAddressError("IPv4")

    def get_next_ipv6(self) -> IPv6Address | None:
        if not self.ipv6_network:
            return None

        used = self.get_used_ipv6_addresses()
        for address in self.ipv6_network.hosts():
            if address in used:
                continue
            return address

        raise FreeAddressError("IPv6")

    def serialize(self) -> dict[str, Any]:
        return {
            "version": self.config_version,
            "ipv4_network": str(self.ipv4_network) if self.ipv4_network else None,
            "ipv6_network": str(self.ipv6_network) if self.ipv6_network else None,
            "default_port": self.default_port,
            "peers": [peer.serialize() for peer in self.peers],
            "point_to_point": [p2p.serialize() for p2p in self.point_to_point],
            "sites": [site.serialize() for site in self.sites],
        }

    @staticmethod
    def deserialize(data: dict[str, Any]) -> MainConfigBase:
        version = int(data["version"])
        if version > CURRENT_CONFIG_VERSION:
            raise ConfigVersionError(version, CURRENT_CONFIG_VERSION)

        if version < CURRENT_CONFIG_VERSION:
            data = MainConfigBase.migrate(data)

        config = MainConfigBase(
            int(data["default_port"]),
            IPv4Network(data["ipv4_network"]) if data["ipv4_network"] else None,
            IPv6Network(data["ipv6_network"]) if data["ipv6_network"] else None,
        )
        config.peers = [PeerConfig.deserialize(entry) for entry in data["peers"]]
        config.point_to_point = [
            PointToPointConfig.deserialize(entry) for entry in data["point_to_point"]
        ]
        config.sites = [Site.deserialize(entry) for entry in data["sites"]]

        return config



from __future__ import annotations

import logging
from enum import Enum
from ipaddress import IPv4Address, IPv6Address

from wgmgr import keygen
from wgmgr.base import MainConfigBase
from wgmgr.error import DuplicatePeerError, UnknownPeerError
from wgmgr.peer import PeerConfig
from wgmgr.templates import get_template
from wgmgr.util import AssignableIPv4, AssignableIPv6, AssignablePort

LOGGER = logging.getLogger(__name__)


def add_peer(
    self: MainConfigBase,
    name: str,
    ipv4: IPv4Address | None = None,
    ipv6: IPv6Address | None = None,
    port: int | None = None,
):
    try:
        self.get_peer(name)
        raise DuplicatePeerError(name)
    except UnknownPeerError:
        pass

    private_key = keygen.generate_private_key()
    peer = PeerConfig(name, private_key, keygen.generate_public_key(private_key))

    if ipv4:
        peer.ipv4 = AssignableIPv4(ipv4, False)
    else:
        if ipv4 := self.get_next_ipv4():
            LOGGER.info("assign IPv4 address %s to peer %s", str(ipv4), peer.name)
            peer.ipv4 = AssignableIPv4(ipv4, True)

    if ipv6:
        peer.ipv6 = AssignableIPv6(ipv6, False)
    else:
        if ipv6 := self.get_next_ipv6():
            LOGGER.info("assign IPv6 address %s to peer %s", str(ipv6), peer.name)
            peer.ipv6 = AssignableIPv6(ipv6, True)

    if port:
        peer.port = AssignablePort(port, False)
    else:
        peer.port = AssignablePort(self.default_port, True)

    self.peers.append(peer)


def remove_peer(self: MainConfigBase, name: str):
    self.peers.remove(self.get_peer(name))


class PeerConfigType(str, Enum):
    wg_quick = "wg-quick"


def generate_peer_config(
    self: MainConfigBase, name: str, config_type: PeerConfigType
) -> str:
    if config_type == PeerConfigType.wg_quick:
        return get_template("wg-quick.conf.j2").render(config=self, peer_name=name)

    raise ValueError(f"Unknown peer config type {config_type}")



from pathlib import Path
from typing import Optional

from typer import Argument, Option, Typer, echo

from wgmgr import MainConfig
from wgmgr.cli import common

app = Typer()


@app.command()
def add(
    peer1: str = Argument(..., help="Name of one peer."),
    peer2: str = Argument(..., help="Name of the other peer."),
    endpoint1: Optional[str] = Option(
        None, help="Endpoint address for peer2 to reach peer1."
    ),
    endpoint2: Optional[str] = Option(
        None, help="Endpoint address for peer1 to reach peer2."
    ),
    config_path: Path = common.OPTION_CONFIG_PATH,
):
    """
    Add a new point-to-point connection.
    """
    config = MainConfig.load(config_path)
    config.add_p2p(peer1, peer2, endpoint1, endpoint2)
    config.save(config_path)


@app.command()
def list(
    config_path: Path = common.OPTION_CONFIG_PATH,
    verbose: bool = Option(False, "-v", "--verbose"),
):
    """
    List point-to-point connections
    """
    config = MainConfig.load(config_path)
    for p2p in config.point_to_point:
        if verbose:
            echo(p2p.serialize())
        else:
            echo(f"{p2p.peer1_name} ↔ {p2p.peer2_name}")



from __future__ import annotations

from typing import Any

from wgmgr.util import AssignableIPv4, AssignableIPv6, AssignablePort


class PeerConfig:
    def __init__(
        self, name: str, private_key: str, public_key: str, site: str | None = None
    ):
        self.name: str = name
        self.private_key: str = private_key
        self.public_key: str = public_key
        self.ipv4: AssignableIPv4 | None
        self.ipv6: AssignableIPv6 | None
        self.port: AssignablePort
        self.site: str | None

    def serialize(self) -> dict[str, Any]:
        return {
            "name": self.name,
            "private_key": self.private_key,
            "public_key": self.public_key,
            "ipv4": self.ipv4.serialize() if self.ipv4 else None,
            "ipv6": self.ipv6.serialize() if self.ipv6 else None,
            "port": self.port.serialize(),
        }

    @staticmethod
    def deserialize(data: dict[str, Any]) -> PeerConfig:
        config = PeerConfig(data["name"], data["private_key"], data["public_key"])
        config.ipv4 = AssignableIPv4.deserialize(data["ipv4"]) if data["ipv4"] else None
        config.ipv6 = AssignableIPv6.deserialize(data["ipv6"]) if data["ipv6"] else None
        config.port = AssignablePort.deserialize(data["port"])
        return config



import logging
from ipaddress import IPv4Network, IPv6Network
from pathlib import Path
from typing import Optional

from typer import Option, Typer, echo

from wgmgr import MainConfig
from wgmgr.cli import common, p2p, peer

logging.basicConfig(level=logging.INFO)

app = Typer()
app.add_typer(peer.app, name="peer", help="Manage peers.")
app.add_typer(
    p2p.app, name="p2p", help="Manage point-to-point connections between peers."
)


@app.command()
def new(
    config_path: Path = common.OPTION_CONFIG_PATH,
    ipv4_network: Optional[str] = common.OPTION_IPV4_NETWORK,
    ipv6_network: Optional[str] = common.OPTION_IPV6_NETWORK,
    default_port: Optional[int] = common.OPTION_PORT,
    force: bool = Option(
        False, "-f", "--force", help="Force overwriting of existing config file"
    ),
):
    """
    Create a new empty config file.
    """
    if config_path.exists() and (not force):
        echo(
            f'config file "{config_path}" exists, add -f/--force flag to overwrite',
            err=True,
        )
        exit(1)

    if default_port is None:
        default_port = 51820

    if ipv4_network is None:
        ipv4_network = "10.0.0.0/24"

    if ipv6_network is None:
        ipv6_network = "fd00:641:c767:bc00::/64"

    config = MainConfig(
        default_port, IPv4Network(ipv4_network), IPv6Network(ipv6_network)
    )
    config.save(config_path)


@app.command()
def set(
    config_path: Path = common.OPTION_CONFIG_PATH,
    port: Optional[int] = common.OPTION_PORT,
    ipv4_network: Optional[str] = common.OPTION_IPV4_NETWORK,
    ipv6_network: Optional[str] = common.OPTION_IPV6_NETWORK,
):
    """
    Change global and default settings.

    If a new IPv4/IPv6 subnet is specified, all automatically assigned IPv4/IPv6
    addresses will be regenerated. Manually set IP addresses will be overwritten
    if they are not contained in the new subnet.

    If a new port is specified, all peers that do not have a manually assigned port
    will be set to use the new default.
    """

    config = MainConfig.load(config_path)
    if port is not None:
        config.set_default_port(port)
    if ipv4_network is not None:
        config.set_ipv4_network(IPv4Network(ipv4_network))
    if ipv6_network is not None:
        config.set_ipv6_network(IPv6Network(ipv6_network))
    config.save(config_path)


@app.command()
def migrate(
    config_path: Path = common.OPTION_CONFIG_PATH,
):
    """
    Load config file, migrate it to the newest version and save it.
    """
    config = MainConfig.load(config_path)
    config.save(config_path)


if __name__ == "__main__":
    app()



from ipaddress import IPv4Address, IPv6Address
from pathlib import Path
from typing import Optional

from typer import Argument, Option, Typer, echo

from wgmgr import MainConfig
from wgmgr.cli import common
from wgmgr.error import DuplicatePeerError, UnknownPeerError
from wgmgr.operations.peer import PeerConfigType

app = Typer()


@app.command()
def add(
    name: str = Argument(..., help="Name of the peer."),
    config_path: Path = common.OPTION_CONFIG_PATH,
    port: Optional[int] = common.OPTION_PORT,
    ipv4_address: Optional[str] = common.OPTION_IPV4_ADDRESS,
    ipv6_address: Optional[str] = common.OPTION_IPV6_ADDRESS,
    site: Optional[str] = None,
):
    """
    Add a new peer.
    """
    config = MainConfig.load(config_path)
    try:
        config.add_peer(
            name,
            IPv4Address(ipv4_address) if ipv4_address else None,
            IPv6Address(ipv6_address) if ipv6_address else None,
            port,
        )
    except DuplicatePeerError as e:
        echo(str(e), err=True)

    config.save(config_path)


@app.command()
def remove(
    name: str = Argument(..., help="Name of the peer."),
    config_path: Path = common.OPTION_CONFIG_PATH,
):
    """
    Remove a new peer.
    """
    config = MainConfig.load(config_path)
    try:
        config.remove_peer(name)
    except UnknownPeerError:
        echo(f"no such peer: {name}", err=True)
    config.save(config_path)


@app.command()
def list(
    config_path: Path = common.OPTION_CONFIG_PATH,
    verbose: bool = Option(False, "-v", "--verbose"),
):
    """
    List peers in the config.
    """
    config = MainConfig.load(config_path)
    for peer in config.peers:
        if verbose:
            echo(peer.serialize())
        else:
            echo(peer.name)


@app.command()
def generate_config(
    name: str = Argument(..., help="Name of the peer."),
    config_type: PeerConfigType = PeerConfigType.wg_quick,
    config_path: Path = common.OPTION_CONFIG_PATH,
):
    """
    Generate config file for a peer.
    """
    config = MainConfig.load(config_path)
    echo(config.generate_peer_config(name, config_type))



from __future__ import annotations

from wgmgr.base import MainConfigBase
from wgmgr.p2p import PointToPointConfig


def add_p2p(
    self: MainConfigBase,
    name1: str,
    name2: str,
    endpoint1: str | None = None,
    endpoint2: str | None = None,
):
    if name1 > name2:
        name1, name2 = name2, name1
        endpoint1, endpoint2 = endpoint2, endpoint1

    if name1 == name2:
        raise ValueError("Cannot add p2p connection between a peer and itself")

    if endpoint1 and (endpoint1 == endpoint2):
        raise ValueError("The peers in a p2p cannot have the same endpoint address")

    for p2p in self.point_to_point:
        if (p2p.peer1_name == name1) and (p2p.peer2_name == name2):
            raise ValueError(
                f"P2P connection between {name1} and {name2} is already present"
            )
        if (p2p.peer1_name == name2) and (p2p.peer2_name == name1):
            raise ValueError(
                f"P2P connection between {name1} and {name2} is already present"
            )

    self.point_to_point.append(PointToPointConfig(name1, name2, endpoint1, endpoint2))



from __future__ import annotations

from ipaddress import IPv4Address, IPv6Address
from typing import Any


class AssignablePort:
    def __init__(self, number: int, auto: bool):
        self.number = number
        self.auto = auto

    def set(self, number: int, auto: bool):
        self.number = number
        self.auto = auto

    def set_auto(self, number: int):
        self.set(number, True)

    def set_manual(self, number: int):
        self.set(number, True)

    def serialize(self) -> dict[str, Any]:
        return {"number": self.number, "auto": self.auto}

    @staticmethod
    def deserialize(data: dict[str, Any]) -> AssignablePort:
        return AssignablePort(int(data["number"]), bool(data["auto"]))


class AssignableIPv4:
    def __init__(self, address: IPv4Address, auto: bool):
        self.address = address
        self.auto = auto

    def set(self, address: IPv4Address, auto: bool):
        self.address = address
        self.auto = auto

    def set_auto(self, address: IPv4Address):
        self.set(address, True)

    def set_manual(self, address: IPv4Address):
        self.set(address, False)

    def serialize(self) -> dict[str, Any]:
        return {"address": str(self.address), "auto": self.auto}

    @staticmethod
    def deserialize(data: dict[str, Any]) -> AssignableIPv4:
        return AssignableIPv4(IPv4Address(data["address"]), bool(data["auto"]))


class AssignableIPv6:
    def __init__(self, address: IPv6Address, auto: bool):
        self.address = address
        self.auto = auto

    def set(self, address: IPv6Address, auto: bool):
        self.address = address
        self.auto = auto

    def set_auto(self, address: IPv6Address):
        self.set(address, True)

    def set_manual(self, address: IPv6Address):
        self.set(address, False)

    def serialize(self) -> dict[str, Any]:
        return {"address": str(self.address), "auto": self.auto}

    @staticmethod
    def deserialize(data: dict[str, Any]) -> AssignableIPv6:
        return AssignableIPv6(IPv6Address(data["address"]), bool(data["auto"]))



from __future__ import annotations

from typing import Any


class Site:
    def __init__(self, name: str):
        self.name = name

    def serialize(self) -> dict[str, Any]:
        return {"name": self.name}

    @staticmethod
    def deserialize(data: dict[str, Any]) -> Site:
        return Site(data["name"])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wgmgr',
 'wgmgr.cli',
 'wgmgr.migrations',
 'wgmgr.operations',
 'wgmgr.templates']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=2.11.3,<4.0.0', 'PyYAML>=5.4.1,<6.0.0', 'typer[all]>=0.3.2,<0.4.0']

entry_points = \
{'console_scripts': ['wgmgr = wgmgr.cli:app']}

setup_kwargs = {
    'name': 'wgmgr',
    'version': '0.3.0',
    'description': 'Easily manage wireguard configs.',
    'long_description': '# wgmgr\n',
    'author': 'Fabian Köhler',
    'author_email': 'fabian.koehler@protonmail.ch',
    'maintainer': 'Fabian Köhler',
    'maintainer_email': 'fabian.koehler@protonmail.ch',
    'url': 'https://github.com/f-koehler/wgmgr/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
